/*
 * Title            :  GBBDriver.java
 * Package      	:  examples
 * Project        	:  Gridbus Broker
 * Description   	:		
 * Created on   	:  March 31, 2005
 * Author	     	:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright     	:   (c) 2005, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.broker.examples.portlets.gridbusbroker;

import java.io.IOException;
import java.util.Vector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.gridbus.broker.farming.common.GridbusFarmingEngine;
import org.gridbus.broker.farming.common.Job;
import org.gridbus.broker.runfile.CopyCommand;
import org.gridbus.broker.runfile.ExecuteCommand;
import org.gridbus.broker.runfile.Task;
import org.gridbus.broker.runfile.TaskCommand;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;


/**
 * This portlet allows you to execute a command over the grid using the Gridbus Broker.
 * 
 * Here we make use of the broker's API functionality to create jobs and then submit them
 * onto the Grid.  The main code for starting up the broker and the jobs here is quite 
 * simple and is easily modified to suit your own needs.
 *
 */
public class GBBDriver extends GenericPortlet {
	
     /**
     * @see javax.portlet.GenericPortlet#processAction(javax.portlet.ActionRequest, javax.portlet.ActionResponse)
     */
    public void processAction(ActionRequest req, ActionResponse res) throws PortletException, IOException {

    	// get the session
    	PortletSession session = req.getPortletSession(true);
    	
    	// create a new farming engine if one doesnt already exist
    	GridbusFarmingEngine fe = (GridbusFarmingEngine) session.getAttribute("fe");
    	if (fe == null) {
    		try {
		    	fe = new GridbusFarmingEngine();
    		} catch (Exception e) {
    		}
	    	session.setAttribute("fe",fe,PortletSession.APPLICATION_SCOPE);
    	}
    	
    	// get our proxy
        GSSCredential credential = (GSSCredential) session.getAttribute("userCred");
        // give the broker the user's proxy so that it can submit jobs on their behalf
        //BrokerProperties.setProxy(credential); //the proxy is set in the resource-portlet now
        //since we need to use "UserCredentials" API to set the proxy

        if (credential != null) {

        	String command = req.getParameter("command");
	        String iterations = req.getParameter("iterations");
	        String action = req.getParameter("action");

	        if (action.compareTo("run") == 0) {
	
		        Vector jobs = null;
		        
		        if (command==null || command.equals(""))
		        	return;
		        
		        if (iterations==null || iterations.equals("") || iterations.equals("0"))
		            return;
		        
		        try {
		            // create the jobs based on the command and number of iterations
		            // specified by the user.
		            jobs = createJobs(command, Integer.parseInt(iterations));
		            fe.setJobs(jobs);
		            
		            if (session.getAttribute("qosSet")!=null && session.getAttribute("qosSet").equals("true")){
		            // schedule our jobs for execution on the Grid!
		                fe.schedule();
		                res.setRenderParameter("qosMsg", "");
		            }else{
		                res.setRenderParameter("qosMsg", "Please set the Qos parameters before running the broker.");
		                return;
		            }
		        } catch (Exception e) {
		            e.printStackTrace();
		        }
		        // must be passed into render parameters
		        res.setRenderParameter("command", command);
		        res.setRenderParameter("iterations", iterations);
		        if (fe != null) {
		        	session.setAttribute("fe", fe, PortletSession.APPLICATION_SCOPE);
		        	res.setRenderParameter("running","true");
		        }
	        } else if (action.compareTo("reset") == 0) {
	        	fe.terminate();
	        	fe = null;
		        session.setAttribute("fe", fe, PortletSession.APPLICATION_SCOPE);
		        res.setRenderParameter("running","false");
	        }
        }
    }

    /**
     * Set up any required view parameters.
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {

    	// get proxy
        PortletSession session = request.getPortletSession(true);
        GSSCredential credential = (GSSCredential) session.getAttribute("userCred");
        String user = "";
        if (credential != null) {
        	try {
        		user = credential.getName().toString();
        	} catch (GSSException e) {
            	e.printStackTrace();
            } 
        }
        
		if (request.getParameter("qosMsg")!=null){
		    request.setAttribute("qosMsg",request.getParameter("qosMsg"));
		}else{
		    request.setAttribute("qosMsg","");
		}
        
        request.setAttribute("userCredName", user);
        request.setAttribute("command", request.getParameter("command"));
        request.setAttribute("iterations", request.getParameter("iterations"));
        request.setAttribute("running", request.getParameter("running"));
        request.setAttribute("url", response.createActionURL().toString());

        getPortletContext().getRequestDispatcher("/jsp/driver.jsp").include(request, response);
    }

    /**
     * Creates the jobs to be given to the broker for execution.
     * @param command
     * @param iterations
     * @return
     */
	private Vector createJobs(String command, int iterations) {

		Vector jobs = new Vector();
		Job j = null;
		
		for (int i = 0; i < iterations; i++) {

			j = new Job();
			j.setJobID(""+i);

			// here we tell the broker to execute "command" on a remote node.
			ExecuteCommand exec = new ExecuteCommand(TaskCommand.EXECUTE_CMD);
			exec.setExecutable(command);

			// after executing "command" we want to copy the "stdout.*" (standard output)
			// on the remote node to a file called "output.*" on our local machine.
			CopyCommand copyOut = new CopyCommand();
			copyOut.setSource(false,"stdout."+i,true);
			copyOut.setDestination(true,"output."+i,true);

			Task task = new Task();
			task.addCommand(exec);
			task.addCommand(copyOut);

			j.setTask(task);
			jobs.add(j);
		}

		return jobs;
	}
}
