/*
 * Title            :  GBBDriverUL.java
 * Package      	:  examples
 * Project        	:  Gridbus Broker
 * Description   	:		
 * Created on   	:  March 31, 2005
 * Author	     	:  Tianchi Ma (tcma@cs.mu.oz.au)
 * Copyright     	:   (c) 2005, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.broker.examples.portlets.gridbusbroker;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.List;
import java.util.Vector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.portlet.PortletFileUpload;
import org.gridbus.broker.farming.common.GridbusFarmingEngine;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;

/**
 * This portlet allows you to execute an application over the grid using the Gridbus Broker.
 * 
 * Here we make use of the broker's API functionality to create jobs and then submit them
 * onto the Grid.  The main code for starting up the broker and the jobs here is quite 
 * simple and is easily modified to suit your own needs.
 *
 */
public class GBBDriverUL extends GenericPortlet {
	
	/**
	 * @see javax.portlet.GenericPortlet#processAction(javax.portlet.ActionRequest, javax.portlet.ActionResponse)
	 */
	
	public void processAction(ActionRequest req, ActionResponse res) throws PortletException, IOException {
		
		//PrintStream ps=new PrintStream(new FileOutputStream("c:\\debug"));
		
		try{
			// get the session
			PortletSession session = req.getPortletSession(true);
			// get our proxy
			GSSCredential credential = (GSSCredential) session.getAttribute("userCred");
			if (credential == null){ 
				throw new Exception("Invalid Credential");
			}

			// create a new farming engine if one doesnt already exist
			GridbusFarmingEngine fe = (GridbusFarmingEngine) session.getAttribute("fe");
			if (fe == null) {
				try {
					fe = new GridbusFarmingEngine();
				} catch (Exception e) {
				}
				session.setAttribute("fe",fe,PortletSession.APPLICATION_SCOPE);
			}

			//	try{
			String t=req.getContentType();
			// 			ps.println(t); 
			// 			ps.println(req.getContentLength());
			// 			ps.println(req.toString());
			DiskFileItemFactory df=new DiskFileItemFactory();
			df.setSizeThreshold(1000);
			
			/***********************************************
			 * Below the PortletFileUpload is come from the library common-fileupload-dev-1.1....
			 * It and its supporting lib common-io-... are currently in the portletlib directory.
			 * Please also copy them into your tomcat shared/lib so that they could be 
			 * visited by the web application
			 */
			
			PortletFileUpload myupload = new PortletFileUpload(df); 
			
			//		ps.println("ok1 "+System.getProperty("java.io.tmpdir"));
			//   	List fileItems=myupload.parseRequest(req);
			List fileItems=myupload.parseRequest(req,req.getContentType(),req.getContentLength(),req.getPortletInputStream());
			//ps.println("ok2");
			int actionIndex=-1;
			int inputfileIndex=-1;
			for(int i=0;i<fileItems.size();i++){
				FileItem tmpfile=(FileItem)fileItems.get(i);
				//ps.println("Index="+i+" fieldname="+tmpfile.getFieldName());
				if(tmpfile.getFieldName().equals("action"))
					actionIndex=i;
				if(tmpfile.getFieldName().equals("inputfile"))
					inputfileIndex=i;
				if(tmpfile.getFieldName().equals("xpmlinput"))
					session.setAttribute("xpml",((FileItem)fileItems.get(i)).getString(), PortletSession.PORTLET_SCOPE);
			}
			//		}catch(Exception e){e.printStackTrace(ps);}
			//ps.println("ok3");
			//ps.close();
			
			
			//    	String action = req.getParameter("action");
			String action=((FileItem)fileItems.get(actionIndex)).getString();
			//ps.println(action);
			actionIndex=-1;
			
			if (action.compareTo("Upload XPML") == 0) {
				
				try{
					
					//    		   	PortletFileUpload myupload = new PortletFileUpload(new DiskFileItemFactory());
					//   		   	List fileItems=myupload.parseRequest(req);
					for(int i=0;i<fileItems.size();i++){
						FileItem tmpfile=(FileItem)fileItems.get(i);
						// 		    	ps.println("Index="+i+" fieldname="+tmpfile.getFieldName());
						if((!tmpfile.isFormField())&&tmpfile.getFieldName().equals("ulxpml")){
							session.setAttribute("xpmlfile",tmpfile.getName(),PortletSession.PORTLET_SCOPE);
							//ps.println("local filename="+ tmpfile.getName());
							BufferedReader br=new BufferedReader(new InputStreamReader(tmpfile.getInputStream()));
							String tmpstr="";
							String line=null;
							while((line=br.readLine())!=null)
								tmpstr=tmpstr+line+"\n";
							//    		    		ps.println("xpml:");
							//    		    		ps.print(tmpstr);
							//    		    		ps.println("");
							br.close();
							
							Vector localfilelist=(Vector)session.getAttribute("localfilelist");
							
							if((localfilelist!=null)&&(localfilelist.size()>0)){
								for(int j=0;j<localfilelist.size();j++){
									String tmp=(String)localfilelist.get(j);
									String debugstr=tmp.replace('\\','/').replaceAll("/","\\\\\\\\");
									tmpstr=tmpstr.replaceAll("upload://"+(new File(tmp)).getName(),debugstr);
								}
							}
							session.setAttribute("xpml", tmpstr,PortletSession.PORTLET_SCOPE);
							session.setAttribute("defaultXpml",null,PortletSession.PORTLET_SCOPE);
						}
					}
					
				}catch(Exception e){e.printStackTrace();}
				//ps.close();
				return;
			}
			
			if (action.compareTo("Upload") == 0) {
				try{
					
					String localdirname=(String)session.getAttribute("localdir",PortletSession.PORTLET_SCOPE);
					//ps.println("localdirname="+localdirname);
					if(localdirname==null){
						//        	    		(new File(localdirname=System.getProperty("java.io.tmpdir")+"\\tmptask"+(new Random(0)).nextInt())).mkdirs();
						//            			localdirname=System.getProperty("java.io.tmpdir")+"/"+fe.getBrokerID();
						localdirname=getPortletContext().getRealPath("/files/input")+"/"+fe.getBrokerID();
						(new File(getPortletContext().getRealPath("/files/input")+"/"+fe.getBrokerID())).mkdirs();
						
						session.setAttribute("localdir",localdirname, PortletSession.PORTLET_SCOPE);       // no the last "\"
					}
					
					Vector localfilelist=(Vector)session.getAttribute("localfilelist",PortletSession.PORTLET_SCOPE);
					if(localfilelist==null)
						localfilelist=new Vector(); 
					
					Vector inputfilelist=(Vector)session.getAttribute("inputfilelist",PortletSession.PORTLET_SCOPE);
					if(inputfilelist==null)
						inputfilelist=new Vector();
					
					String inputfiles=(String)session.getAttribute("inputfiles");
					/*
					 if(inputfileIndex!=-1){
					 inputfiles=((FileItem)fileItems.get(inputfileIndex)).getString();
					 if(inputfiles==null)
					 inputfiles="";
					 }else{
					 inputfiles="";
					 }
					 inputfileIndex=-1;
					 */
					for(int i=0;i<fileItems.size();i++){
						FileItem tmpfile=(FileItem)fileItems.get(i);
						//ps.println("Index="+i+" fieldname="+tmpfile.getFieldName());
						if((!tmpfile.isFormField())&&tmpfile.getFieldName().equals("ulinput")){
							String tmpname=localdirname+"/"+(new File(tmpfile.getName())).getName();
							//ps.println("tmpname="+tmpname);
							//ps.println("inputfilename="+(new File(tmpfile.getName())).getName());
							File f=new File(tmpname);
							tmpfile.write(f);
							localfilelist.add(tmpname);
							inputfilelist.add((new File(tmpfile.getName())).getName());
							session.setAttribute("inputfilelist",inputfilelist,PortletSession.PORTLET_SCOPE);
							session.setAttribute("localfilelist",localfilelist,PortletSession.PORTLET_SCOPE);
							if(!inputfiles.equals(""))
								inputfiles=inputfiles+",";
							inputfiles=inputfiles+(new File(tmpfile.getName())).getName();
							//ps.println(localfilelist.size());
							//ps.println(inputfiles);
							
							String pattern="upload://"+(new File(tmpfile.getName())).getName();
							String newxpml=((String)session.getAttribute("xpml"));			    		
							
							String debugstr="";
							try{
								debugstr=tmpname.replace('\\','/').replaceAll("/","\\\\\\\\");
								//ps.println("debugstr="+debugstr);
								newxpml=newxpml.replaceAll(pattern,debugstr);
							}catch(Exception e){newxpml=debugstr+"<br>"+e.getMessage();}
							session.setAttribute("xpml",newxpml,PortletSession.PORTLET_SCOPE);
						}
					}
					
					session.setAttribute("inputfiles", inputfiles,PortletSession.PORTLET_SCOPE);
					//refresh the upload
					//upload=new SmartUpload();
				}catch(Exception e){e.printStackTrace();}
				return;
			}
			
			if (action.compareTo("Delete All Input Files") == 0) {
				Vector localfilelist=(Vector)session.getAttribute("localfilelist");
				String myxpml=(String)session.getAttribute("xpml");
				if((localfilelist!=null)&&(localfilelist.size()>0)){
					for(int i=0;i<localfilelist.size();i++){
						String tmpstr=(String)localfilelist.get(i);
						String debugstr=tmpstr.replace('\\','/').replaceAll("/","\\\\\\\\");
						myxpml=myxpml.replaceAll(debugstr,"upload://"+(new File(tmpstr)).getName());
					}
					session.setAttribute("xpml",myxpml,PortletSession.PORTLET_SCOPE);
				}
				
				session.setAttribute("localdir",null,PortletSession.PORTLET_SCOPE);
				session.setAttribute("inputfilelist",null,PortletSession.PORTLET_SCOPE);
				session.setAttribute("localfilelist",null,PortletSession.PORTLET_SCOPE);
				session.setAttribute("inputfiles","",PortletSession.PORTLET_SCOPE);
				return;
				
			}
			
			if (action.compareTo("Stop/Reset Experiment") == 0) {
				session.setAttribute("localdir",null,PortletSession.PORTLET_SCOPE);
				session.setAttribute("inputfilelist",null,PortletSession.PORTLET_SCOPE);
				session.setAttribute("localfilelist",null,PortletSession.PORTLET_SCOPE);
				session.setAttribute("inputfiles","",PortletSession.PORTLET_SCOPE);
				session.setAttribute("init",null,PortletSession.PORTLET_SCOPE);
				session.setAttribute("xpml","",PortletSession.PORTLET_SCOPE);
				session.setAttribute("xpmlfile","",PortletSession.PORTLET_SCOPE);
				session.setAttribute("localdir",null,PortletSession.PORTLET_SCOPE);
			
				if(fe!=null){
					try{
						fe.terminate();
					}catch (Exception e){ }  //do nothing, just try to terminate the fe
				}
				
				fe = null;
				session.setAttribute("fe", fe, PortletSession.APPLICATION_SCOPE);
				session.removeAttribute("fe");
				res.setRenderParameter("running","false");
			}
										
			if (action.compareTo("Use Default") == 0) {		
				try{
					StringBuffer xpml = new StringBuffer();
					String xpmlfile = getPortletContext().getRealPath("/sample_files/xpml.xml");
					BufferedReader br=new BufferedReader(new FileReader(xpmlfile));
					String temp=null;
					while ((temp=br.readLine())!=null)
						xpml.append(temp+"\n");
					
					session.setAttribute("defaultXpml","true",PortletSession.PORTLET_SCOPE);
					session.setAttribute("xpml",xpml.toString(),PortletSession.PORTLET_SCOPE);
					session.setAttribute("xpmlfile",xpmlfile,PortletSession.PORTLET_SCOPE);
					session.setAttribute("inputfiles","calc",PortletSession.PORTLET_SCOPE);
				}catch (Exception e){
					e.printStackTrace();
				}
			}
			if (action.compareTo("Start Experiment") == 0) {			
				try {
					String localdir=null;
					if (session.getAttribute("defaultXpml")==null || !session.getAttribute("defaultXpml").equals("true")){
						String myxpml=(String)session.getAttribute("xpml");					
						if (myxpml==null || myxpml.equals(""))
							return;
						if(myxpml.indexOf("upload://")!=-1)
							return;
						localdir=(String)session.getAttribute("localdir");
						PrintStream myps=new PrintStream(new FileOutputStream(localdir+"/xpml.xml"));
						myps.println(myxpml);
						myps.close();						
					}else{
						localdir = getPortletContext().getRealPath("/sample_files");
					}

					//this is where the broker will get its inputs from
					//BrokerProperties.setProperty("LOCALDIR",localdir);
					fe.setAppDescriptionFile(localdir+"/xpml.xml");
					fe.initJobs();
					
					//copy the input files to the broker's local dir?
					File[] files = new File(localdir).listFiles();
		        	for (int i = 0; i < files.length; i++) {
		        	    try{
			        		File newFile = new File(new File(System.getProperty("user.dir")).getAbsolutePath()+"/"+files[i].getName());
			        		BufferedInputStream bis = new BufferedInputStream(new FileInputStream(files[i]));
			        		BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(newFile));
			        		int temp = 0;
			        		while ((temp=bis.read())!=-1)
			        		    bos.write(temp);
			        		bis.close();bos.close();
			        		//dont move the files anymore. just copy as above
		        	    }catch (Exception e){}
		        		//ps.println("copying "+files[i]+" to "+newFile);
		        	}
		        	
		            if (session.getAttribute("qosSet")!=null && session.getAttribute("qosSet").equals("true")){
			            // schedule our jobs for execution on the Grid!
			                fe.schedule();
			                res.setRenderParameter("qosMsg", "");
			            }else{
			                res.setRenderParameter("qosMsg", "Please set the Qos parameters before running the broker.");
			                return;
			            }
		        } catch (Exception e) {
					e.printStackTrace();
				}
				if (fe != null) {
					session.setAttribute("fe", fe, PortletSession.APPLICATION_SCOPE);
					res.setRenderParameter("running","true");
				}
			} 

		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	/**
	 * Set up any required view parameters.
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
	public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
		PortletSession session = request.getPortletSession(true);
		try{
			String init=(String)session.getAttribute("init");
			if((init==null)||(init.equals("true"))){
				session.setAttribute("xpmlfile","",PortletSession.PORTLET_SCOPE);
				session.setAttribute("xpml","",PortletSession.PORTLET_SCOPE);
				request.setAttribute("running","false");
				session.setAttribute("inputfiles","",PortletSession.PORTLET_SCOPE);
				session.setAttribute("init","false",PortletSession.PORTLET_SCOPE);
				session.setAttribute("localdir",null,PortletSession.PORTLET_SCOPE);
				session.setAttribute("inputfilelist",null,PortletSession.PORTLET_SCOPE);
				session.setAttribute("localfilelist",null,PortletSession.PORTLET_SCOPE);
				session.setAttribute("defaultXpml",null,PortletSession.PORTLET_SCOPE);
			}
			
			// get proxy
			GSSCredential credential = (GSSCredential) session.getAttribute("userCred");
			String user = "";
			if (credential != null) {
				try {
					user = credential.getName().toString();
				} catch (GSSException e) {
					e.printStackTrace();
				} 
			}
			
			if (request.getParameter("qosMsg")!=null){
			    request.setAttribute("qosMsg",request.getParameter("qosMsg"));
			}else{
			    request.setAttribute("qosMsg","");
			}

			request.setAttribute("userCredName", user);
			
			request.setAttribute("xpmlfile",session.getAttribute("xpmlfile"));
			request.setAttribute("xpml",session.getAttribute("xpml"));
			request.setAttribute("inputfiles",session.getAttribute("inputfiles"));       
			request.setAttribute("running", request.getParameter("running"));
			request.setAttribute("url", response.createActionURL().toString());
			try{
				getPortletContext().getRequestDispatcher("/jsp/driverUL.jsp").include(request, response);
			}catch(Exception e){
				e.printStackTrace();
			}
		}catch(Exception e){
			e.printStackTrace();
		}
	}
}
