/*
 * Title            :  GBBMonitor.java
 * Package      	:  examples
 * Project        	:  Gridbus Broker
 * Description   	:		
 * Created on   	:  March 31, 2005
 * Author	     	:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright     	:   (c) 2005, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.broker.examples.portlets.gridbusbroker;

import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.gridbus.broker.farming.common.BrokerProperties;
import org.gridbus.broker.farming.common.GridbusFarmingEngine;
import org.gridbus.broker.farming.common.Job;
import org.gridbus.broker.util.BrokerUtil;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;


/**
 * A simple portlet for displaying the execution status of the Gridbus Broker.  This
 * really just passes the set of jobs to the view where each of the job's statuses are
 * printed.
 */
public class GBBMonitor extends GenericPortlet {

	/**
	 * @param req
	 * @param res
	 * @throws PortletException
	 * @throws IOException
	 */
    public void processAction(ActionRequest req, ActionResponse res) throws PortletException, IOException {

        // get the session
    	PortletSession session = req.getPortletSession(true);
        String action = req.getParameter("action");
        if (action!=null && action.equals("view")) {
        	session.setAttribute("jid",req.getParameter("jid"),PortletSession.APPLICATION_SCOPE);
        	//ps.println("action is view, jid="+req.getParameter("jid"));
        } else {
            //ps.println("action is not view:"+action);
        }
        
        if (req.getParameter("txtRefreshtime")!=null && !req.getParameter("txtRefreshtime").equals("")){
            session.setAttribute("refreshtime",req.getParameter("txtRefreshtime"));
        }
    }

    /**
     * Get the current status of the Gridbus Broker.  This includes determining whether or not
     * the broker is actually running as well as printing out the statuses of each job
     * (ie. waiting, pending, submitted, failed, done. etc.)
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {

    	// get the session
        PortletSession session = request.getPortletSession(true);
		//PrintStream ps=new PrintStream(new FileOutputStream("c:\\monDebug"));
       
        // get our proxy from the session
        GSSCredential credential = (GSSCredential) session.getAttribute("userCred");
        String user = "";
        String status = "stopped";
        GridbusFarmingEngine fe = null;
        Collection jobs = new Vector();
        Collection servers = new Vector();
        String refreshtime = "10"; //default
        
        if (credential != null) {

            //set defaults, in case nothing is selected
            refreshtime = (String) session.getAttribute("refreshtime");
            if (refreshtime==null || refreshtime.trim().equals("")) refreshtime = "10";

        	try {
        		user = credential.getName().toString();
        	} catch (GSSException e) {
            	e.printStackTrace();
            } 

        	// get the Gridbus Broker farming engine from the session.
        	fe = (GridbusFarmingEngine) session.getAttribute("fe");
        	
	        if (fe != null) {
				// get the current set of jobs
				jobs = fe.getJobs();
				servers = fe.getServers();
				//if (jobs!=null)
					//ps.println("in doView.. jobs size="+jobs.size());
				//else
					//ps.println("in doView.. jobs=NULL ");
				
	        	// check the overall status of the broker 
	        	if (fe.getScheduler()!=null){
					if (fe.isSchedulingFailed()) {
		        		status = "failed";
		        	} else if (fe.isSchedulingFinished()) {
		        		status = "finished";
		        	} else {
		        		status = "running";
		        	}
	        	}
	        	try{
	        		long donejobs=0;
	        		if (jobs!=null){
	        			for (Iterator it=jobs.iterator();it.hasNext();){
	        				Job j = (Job)it.next();
	        				if (j.getJobStatus()==Job.DONE || j.getJobStatus()==Job.FAILED)
	        					donejobs++;
	        			}
	        		}
	        		
	        		long totaljobs=0;
	        		if (jobs!=null)
	        			totaljobs = jobs.size();
	        		float temp = 0;
	        		
	        		temp = 100 * donejobs / totaljobs;
	        		request.setAttribute("jDone", Float.toString(temp));
	        		
	        		temp = 100 * (totaljobs - donejobs) / totaljobs;
	        		request.setAttribute("jRemain", Float.toString(temp));
	        		
					String timerem = "0";
					if (fe.getScheduler()!=null && !fe.isSchedulingComplete()){
					    //only calculcate time-remaining if the broker is running
					    timerem = BrokerUtil.getHHMMSS(fe.getDeadline() - System.currentTimeMillis());
					}
	        		request.setAttribute("timeRemaining", timerem);
	        		
	        		float totbudget = fe.getBudget();
	        		float budspent = Float.parseFloat(BrokerProperties.getProperty("BUDGET_SPENT","0"));
	        		temp = 100 * budspent / totbudget;
	        		request.setAttribute("bDone", Float.toString(temp));
	        		//ps.println("budget tot="+totbudget);
	        		
	        		temp = 100 * (totbudget - budspent) / totbudget;
	        		request.setAttribute("bRemain", Float.toString(temp));
	        		//ps.println("budget spent="+budspent);
	        		//ps.println("budget rem="+(totbudget-budspent));
	        		
	        		request.setAttribute("showProgress", "true");
	        	}catch(Exception e){
	        		//e.printStackTrace();
	        		//ps.println(e.toString());
	        	}
	        }else{
	        	//ps.println("in doView..fe is null ");
	        	request.setAttribute("showProgress", "false");
	        	request.setAttribute("jDone", "0");
	        	request.setAttribute("jRemain", "0");
	        	request.setAttribute("bDone", "0");
	        	request.setAttribute("bRemain", "0");
	        	request.setAttribute("timeRemaining", "0");
	        }
        }
        
        request.setAttribute("status", status);
        request.setAttribute("jobs", jobs);
        request.setAttribute("servers", servers);
        request.setAttribute("url", response.createActionURL().toString());
        request.setAttribute("userCredName", user);
        request.setAttribute("refreshtime",refreshtime);
    	
        getPortletContext().getRequestDispatcher("/jsp/monitor.jsp").include(request, response);
    }
}
