/*
 * Title            :  GBBDriver.java
 * Package      	:  examples
 * Project        	:  Gridbus Broker
 * Description   	:		
 * Created on   	:  March 31, 2005
 * Author	     	:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright     	:   (c) 2005, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.broker.examples.portlets.gridbusbroker;

import java.io.IOException;
import java.util.GregorianCalendar;
import java.util.Locale;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.gridbus.broker.farming.common.BrokerProperties;
import org.gridbus.broker.farming.common.GridbusFarmingEngine;
import org.gridbus.broker.scheduler.DBScheduler;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;

/**
 * This portlet allows you to select required scheduling algorithm and Qos settings for the application,
 * run by the Gridbus Broker.
 *
 */
public class GBBQos extends GenericPortlet {
	
     /**
     * @see javax.portlet.GenericPortlet#processAction(javax.portlet.ActionRequest, javax.portlet.ActionResponse)
     */
    public void processAction(ActionRequest req, ActionResponse res) throws PortletException, IOException {

    	String qosMsg = "";
    	
    	// get the session
    	PortletSession session = req.getPortletSession(true);
    	//PrintStream ps=new PrintStream(new FileOutputStream("c:\\qosDebug"));
    	
    	// get our proxy
    	GSSCredential credential = (GSSCredential) session.getAttribute("userCred");
    	if (credential != null) {
    		
    		String hh = req.getParameter("selHour");
    		String mm = req.getParameter("selMin");
    		String ss = req.getParameter("selSec");
    		String dd = req.getParameter("selDay");
    		String mmm = req.getParameter("selMonth");
    		String yyyy = req.getParameter("selYear");
    		String deadline = dd+" "+mmm+" "+yyyy+" "+hh+":"+mm+":"+ss;
    		//ps.println("in processAction.."+deadline);
    		
    		String budget = req.getParameter("txtBudget");
    		String schedAlgo = req.getParameter("selAlgo");
    		// get the Gridbus Broker farming engine from the session.
    		GridbusFarmingEngine fe = (GridbusFarmingEngine) session.getAttribute("fe");
    		
    		if (fe!=null){
				if (req.getParameter("setAlgo")!=null && req.getParameter("setAlgo").compareTo("Set Algorithm")==0) {
					//Set Algo button clicked
					qosMsg = setAlgo(fe,schedAlgo);
				}else if (req.getParameter("setDeadline")!=null && req.getParameter("setDeadline").compareTo("Set Deadline")==0) {
					//Set Deadline button clicked
					qosMsg = setDeadline(fe,deadline);
				}else if (req.getParameter("setBudget")!=null && req.getParameter("setBudget").compareTo("Set Budget")==0) {
					//Set Budget button clicked
					qosMsg = setBudget(fe,budget);
				}else if (req.getParameter("setAll")!=null && req.getParameter("setAll").compareTo("Set QoS parameters")==0){
					qosMsg = setAlgo(fe,schedAlgo);
					qosMsg += "<br />" + setDeadline(fe,deadline);
					qosMsg += "<br />" + setBudget(fe,budget);
				}
				session.setAttribute("qosSet","true",PortletSession.APPLICATION_SCOPE);
    		}else{
    			qosMsg = "The broker is not initialised. QoS parameters not set.";
    			session.setAttribute("qosSet","false",PortletSession.APPLICATION_SCOPE);
    		}
    		
	        // must be passed into render parameters
	        res.setRenderParameter("HH", hh);
	        res.setRenderParameter("MM", mm);
	        res.setRenderParameter("SS", ss);
	        res.setRenderParameter("DD", dd);
	        res.setRenderParameter("MMM", mmm);
	        res.setRenderParameter("YYYY", yyyy);
	        
	        res.setRenderParameter("budget", budget);
	        res.setRenderParameter("schedAlgo", schedAlgo);
	        
	        res.setRenderParameter("qosMsg",qosMsg);
        }
    }
    
    private String setAlgo(GridbusFarmingEngine fe,  String schedAlgo){
    	try{
			if (!fe.isSchedulingComplete()){
				//it has either not started, or it has completed.
				//we set it here assuming it has not started yet :)
				BrokerProperties.setProperty("SCHEDULE",schedAlgo);
				return "Algorithm changed successfully.";
			}else{
				//we can change algo only for the DBScheduler, 
				//if it has started
				if (fe.getScheduler()==null || fe.getScheduler() instanceof DBScheduler){
					BrokerProperties.setProperty("SCHEDULE",schedAlgo);
					return "Algorithm changed successfully.";
				}else{
			        return "The scheduling algorithm can be changed for non-data apps only.";
				}
			}
    	}catch(Exception e){
    		return "Couldnot set algorithm: "+e.getMessage();
    	}
    }
    
    private String setDeadline(GridbusFarmingEngine fe,String deadline){
    	try{
			//Set Deadline button clicked
			fe.setDeadline(deadline);
			return "Deadline changed successfully.";
    	}catch(Exception e){
    		return "Couldnot set deadline: "+e.getMessage();
    	}
    }
    
    private String setBudget(GridbusFarmingEngine fe,String budget){
		try{
			//Set Budget button clicked
			fe.setBudget(budget);
			return "Budget changed successfully.";
		}catch (Exception e){
			return "Couldnot set budget: "+e.getMessage();
		}
    }

    /**
     * Set up any required view parameters.
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {

    	// get proxy
        PortletSession session = request.getPortletSession(true);
        GSSCredential credential = (GSSCredential) session.getAttribute("userCred");
        String user = "";
        if (credential != null) {
        	try {
        		user = credential.getName().toString();
        	} catch (GSSException e) {
            	e.printStackTrace();
            } 
        }
        
        //set defaults, in case nothing is selected
        String budget = request.getParameter("budget");
        if (budget==null || budget.trim().equals("")) budget = "10000";
        
        GregorianCalendar gcal = new GregorianCalendar(Locale.ENGLISH);
        gcal.setTimeInMillis(System.currentTimeMillis());

        String HH = request.getParameter("HH");
        if (HH==null || HH.trim().equals("")) 
        	HH = Integer.toString(gcal.get(GregorianCalendar.HOUR_OF_DAY));
        String MM = request.getParameter("MM");
        if (MM==null || MM.trim().equals("")) 
        	MM = Integer.toString(gcal.get(GregorianCalendar.MINUTE));
        String SS = request.getParameter("SS");
        if (SS==null || SS.trim().equals("")) 
        	SS = Integer.toString(gcal.get(GregorianCalendar.SECOND));
        
        String DD = request.getParameter("DD");
        if (DD==null || DD.trim().equals("")) 
        	DD = Integer.toString(gcal.get(GregorianCalendar.DAY_OF_MONTH)+1);
        String MMM = request.getParameter("MMM");
        if (MMM==null || MMM.trim().equals("")) 
        	switch (gcal.get(GregorianCalendar.MONTH)){
        	case GregorianCalendar.JANUARY:
        		MMM = "Jan";
        		break;
        	case GregorianCalendar.FEBRUARY:
        		MMM = "Feb";
        		break;
        	case GregorianCalendar.MARCH:
        		MMM = "Mar";
        		break;
        	case GregorianCalendar.APRIL:
        		MMM = "Apr";
        		break;
        	case GregorianCalendar.MAY:
        		MMM = "May";
        		break;
        	case GregorianCalendar.JUNE:
        		MMM = "Jun";
        		break;        		
        	case GregorianCalendar.JULY:
        		MMM = "Jul";
        		break;
        	case GregorianCalendar.AUGUST:
        		MMM = "Aug";
        		break;
        	case GregorianCalendar.SEPTEMBER:
        		MMM = "Sep";
        		break;
        	case GregorianCalendar.OCTOBER:
        		MMM = "Oct";
        		break;
        	case GregorianCalendar.NOVEMBER:
        		MMM = "Nov";
        		break;
        	case GregorianCalendar.DECEMBER:
        		MMM = "Dec";
        		break;
        	}
        String YYYY = request.getParameter("YYYY");
        if (YYYY==null || YYYY.trim().equals("")) 
        	YYYY = Integer.toString(gcal.get(GregorianCalendar.YEAR));        
        
        request.setAttribute("url", response.createActionURL().toString());
    	request.setAttribute("userCredName", user);        
        request.setAttribute("schedAlgo", request.getParameter("schedAlgo"));
        request.setAttribute("budget", budget);
        request.setAttribute("HH", HH);
        request.setAttribute("MM", MM);
        request.setAttribute("SS", SS);
        request.setAttribute("DD", DD);
        request.setAttribute("MMM", MMM);
        request.setAttribute("YYYY", YYYY);
        request.setAttribute("qosMsg",request.getParameter("qosMsg"));
        
        getPortletContext().getRequestDispatcher("/jsp/qos.jsp").include(request, response);
    }
}
