/*
 * Title            :  GBBReources.java
 * Package      	:  examples
 * Project        	:  Gridbus Broker
 * Description   	:		
 * Created on   	:  March 31, 2005
 * Author	     	:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright     	:   (c) 2005, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.broker.examples.portlets.gridbusbroker;

import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.gridbus.broker.farming.common.ComputeServer;
import org.gridbus.broker.farming.common.GridbusFarmingEngine;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;


/**
 * A simple portlet for managing resources.  Allows you to add resources as well as start and
 * stop them.
 *
 */
public class GBBResourceInfo extends GenericPortlet {
	
	/**
	 * Add / Remove resources here.
	 * @param req
	 * @param res
	 * @throws PortletException
	 * @throws IOException
	 */
	public void processAction(ActionRequest req, ActionResponse res) throws PortletException, IOException {		
	}
	
	/**
	 * Get the current list of resources of the Gridbus Broker.
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
	public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
		
		// get the session
		PortletSession session = request.getPortletSession(true);
		
		// get our proxy from the session
		GSSCredential credential = (GSSCredential) session.getAttribute("userCred");
		String user = "";
		String resourceId = "";
		GridbusFarmingEngine fe = null;
		Collection resources = new Vector();
		ComputeServer resource = null;
		
		if (credential != null) {
			try {
				user = credential.getName().toString();
			} catch (GSSException e) {
				e.printStackTrace();
			} 
			
			// get the Gridbus Broker farming engine from the session.
			fe = (GridbusFarmingEngine) session.getAttribute("fe");
			if (fe == null) {
				try {
					fe = new GridbusFarmingEngine();
				} catch (Exception e) {
				}
				session.setAttribute("fe",fe,PortletSession.APPLICATION_SCOPE);
			}			
			resourceId = (String) session.getAttribute("resourceId");
			if (fe != null && resourceId != null) {
				// get the current set of compute servers
				resources = fe.getServers();
				for (Iterator it=resources.iterator();it.hasNext();) {
					resource = (ComputeServer) it.next();
					if (resource.getServerID().compareTo(resourceId) == 0) {
						request.setAttribute("resource", resource);
						break;
					}
				}
			}
		}
		
		request.setAttribute("url", response.createActionURL().toString());
		request.setAttribute("userCredName", user);
		
		getPortletContext().getRequestDispatcher("/jsp/resourceinfo.jsp").include(request, response);
	}
}
