/*
 * Title            :  GBBReources.java
 * Package      	:  examples
 * Project        	:  Gridbus Broker
 * Description   	:		
 * Created on   	:  March 31, 2005
 * Author	     	:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright     	:   (c) 2005, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.broker.examples.portlets.gridbusbroker;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.portlet.PortletFileUpload;
import org.gridbus.broker.farming.common.ComputeServer;
import org.gridbus.broker.farming.common.GridbusFarmingEngine;
import org.gridbus.broker.farming.common.ServerFactory;
import org.gridbus.broker.farming.common.security.ProxyCredential;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;


/**
 * A simple portlet for managing resources.  Allows you to add resources as well as start and
 * stop them.
 *
 */
public class GBBResources extends GenericPortlet {
	
	/**
	 * Add / Remove resources here.
	 * @param req
	 * @param res
	 * @throws PortletException
	 * @throws IOException
	 */
	public void processAction(ActionRequest req, ActionResponse res) throws PortletException, IOException {
		
		// get the session
		PortletSession session = req.getPortletSession(true);

		// create a new farming engine if one doesnt already exist
		GridbusFarmingEngine fe = (GridbusFarmingEngine) session.getAttribute("fe");
		if (fe == null) {
			try {
				fe = new GridbusFarmingEngine();
			} catch (Exception e) {
			}
			session.setAttribute("fe",fe,PortletSession.APPLICATION_SCOPE);
		}
		
		//tcma added here
		try {  //this try is for debugging
			//PrintStream ps=new PrintStream(new FileOutputStream("c:\\debug"));  //debugging
			String type=req.getContentType();
			//ps.println("contentType="+type);
			if(type.indexOf("multipart/form-data")!=-1){
				DiskFileItemFactory df=new DiskFileItemFactory();
				df.setSizeThreshold(1000);
				PortletFileUpload myupload = new PortletFileUpload(df); 
				
				List fileItems=null;
				try {
					fileItems = myupload.parseRequest(req,req.getContentType(),req.getContentLength(),req.getPortletInputStream());
				} catch (FileUploadException e) {
					e.printStackTrace();
					return;
				} catch (IOException e) {
					e.printStackTrace();
					return;
				}
				int actionIndex=-1;
				int importIndex=-1;
				for(int i=0;i<fileItems.size();i++){
					FileItem tmpfile=(FileItem)fileItems.get(i);
					//ps.println("index "+i+"="+tmpfile.getFieldName());
					if(tmpfile.getFieldName().equals("action"))
						actionIndex=i;
					if(tmpfile.getFieldName().equals("ulresource"))
						importIndex=i;
				}
				
				String localdir=(String)session.getAttribute("localdir");
				if(localdir==null){
					session.setAttribute("localdir",localdir=getPortletContext().getRealPath("/files/input")+"/"+fe.getBrokerID(),PortletSession.PORTLET_SCOPE);
					(new File(getPortletContext().getRealPath("/files/input")+"/"+fe.getBrokerID())).mkdirs();
				}
				//ps.println("localDir="+localdir);
				
				FileItem tmpfile=(FileItem)fileItems.get(importIndex);
				
				String tmpname=localdir+"/"+(new File(tmpfile.getName())).getName();
				//ps.println("tmpname="+tmpname);
				
				File f=new File(tmpname);
				try {
					tmpfile.write(f);
				} catch (Exception e1) {					
					e1.printStackTrace();
					return;
				}
				
				fe.setResourceDescriptionFile(tmpname);
				fe.initResources();
				try{
					//start the servers if they are not started already
					for (Iterator it=fe.getServers().iterator();it.hasNext();){
						ComputeServer cs = (ComputeServer) it.next();
						cs.startup();
					}
				}catch(Exception e){}
				session.setAttribute("fe",fe,PortletSession.APPLICATION_SCOPE);
				return;
			}
			//end of tcma
		} catch(Exception e){}   //for test use
		
		try {
			String action = req.getParameter("action");
			if (action.compareTo("add") == 0) {
				if (req.getParameter("type").compareTo("globus") == 0) {
					if (req.getParameter("host")!=null && req.getParameter("host").trim().equals("")==false){
						ComputeServer cs = ServerFactory.getComputeServer(req.getParameter("host"));
						ProxyCredential pc = new ProxyCredential();
						pc.setProxy((GSSCredential) session.getAttribute("userCred"));
						cs.setUserCredential(pc);
						if (req.getParameter("txtCost")!=null && req.getParameter("txtCost").equals("")==false){
							try{
								cs.setPricePerCpuSec(Float.parseFloat(req.getParameter("txtCost")));
								cs.setPricePerJob(Float.parseFloat(req.getParameter("txtCost")));
							}catch (Exception e){}
						}
						try{
							cs.startup();
						}catch (Exception e){}
						fe.addServer(cs);
					}
				}
			} else if (action.compareTo("Start") == 0) {
				// get the current set of compute servers and set the selected server to alive
				Collection resources = fe.getServers();
				ComputeServer cs = null;
				for (Iterator it=resources.iterator();it.hasNext();) {
					cs = (ComputeServer)it.next();
					if (cs.getServerID().compareTo(req.getParameter("serverID")) == 0) {
						cs.startup();
						break;
					}
				}
			} else if (action.compareTo("Start All") == 0) {
				// get the current set of compute servers and start them all
				try {
					Collection resources = fe.getServers();
					ComputeServer cs = null;
					for (Iterator it=resources.iterator();it.hasNext();) {
						cs = (ComputeServer)it.next();					
						cs.startup();
					}
				}catch (Exception e){}
			}else if (action.compareTo("Stop All") == 0) {
				// get the current set of compute servers and shut them all down
				try {
					Collection resources = fe.getServers();
					ComputeServer cs = null;
					for (Iterator it=resources.iterator();it.hasNext();) {
						cs = (ComputeServer)it.next();					
						cs.shutdown();
					}
				}catch (Exception e){}				
			} else if (action.compareTo("Stop") == 0) {
				// get the current set of compute servers and set the selected server to alive
				Collection resources = fe.getServers();
				ComputeServer cs = null;
				for (Iterator it=resources.iterator();it.hasNext();) {
					cs = (ComputeServer)it.next();
					if (cs.getServerID().compareTo(req.getParameter("serverID")) == 0) {
						cs.shutdown();
						break;
					}
				}
			}else if (action.compareTo("Use Default") == 0){
				try{
					//get the default resources from the local resource.xml file.
					fe.setResourceDescriptionFile(getPortletContext().getRealPath("/sample_files/resources.xml"));
					fe.initResources();
					for (Iterator it=fe.getServers().iterator();it.hasNext();){
						ComputeServer cs = (ComputeServer)it.next();
						cs.startup();
					}
				}catch (Exception e){}
			} else if (action.compareTo("view") == 0) {
				String resourceId = req.getParameter("resourceId");
		    	if (resourceId == null) {
		    		resourceId = (String) session.getAttribute("resourceId");
		    	}
		    	if (resourceId != null) {
		    		// put project into session instead
		    		session.setAttribute("resourceId",resourceId,PortletSession.APPLICATION_SCOPE);
		    	} else {
		    		session.removeAttribute("resourceId",PortletSession.APPLICATION_SCOPE);
		    	}
			}
		}catch(Exception e){}   //for test use
	}
	
	/**
	 * Get the current list of resources of the Gridbus Broker.
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
	public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
		
		// get the session
		PortletSession session = request.getPortletSession(true);
		
		// get our proxy from the session
		GSSCredential credential = (GSSCredential) session.getAttribute("userCred");
		String user = "";
		GridbusFarmingEngine fe = null;
		Collection resources = new Vector();
		
		if (credential != null) {
			try {
				user = credential.getName().toString();
			} catch (GSSException e) {
				e.printStackTrace();
			} 
			
			// get the Gridbus Broker farming engine from the session.
			fe = (GridbusFarmingEngine) session.getAttribute("fe");	
			if (fe != null) {
				// get the current set of compute servers
				resources = fe.getServers();
			}
		}
		
		request.setAttribute("resources", resources);
		request.setAttribute("url", response.createActionURL().toString());
		request.setAttribute("userCredName", user);
		
		getPortletContext().getRequestDispatcher("/jsp/resources.jsp").include(request, response);
	}
}
