/*
 * Title            :  GBBResults.java
 * Package      	:  examples
 * Project        	:  Gridbus Broker
 * Description   	:		
 * Created on   	:  March 31, 2005
 * Author	     	:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright     	:   (c) 2005, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.broker.examples.portlets.gridbusbroker;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.gridbus.broker.farming.common.GridbusFarmingEngine;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;

/**
 * This portlet is used to copy and display the output files of the Gridbus Broker.
 *
 */
public class GBBResults extends GenericPortlet {
	
	/**
	 * Nothing much to do here.
	 * @param req
	 * @param res
	 * @throws PortletException
	 * @throws IOException
	 */
    public void processAction(ActionRequest req, ActionResponse res) throws PortletException, IOException {
    }

	/**
	 * The broker puts its output files in a temporary directory.  Everytime the portlet
	 * refreshes itself, we check the temporary directory for outputs, if any exist we move them
	 * to a directory within our webapp so we can then access and display them.
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
    public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {

    	// get proxy
        PortletSession session = request.getPortletSession(true);
        GSSCredential credential = (GSSCredential) session.getAttribute("userCred");
        String user = "";
        if (credential != null) {
        	try {
        		user = credential.getName().toString();
        	} catch (GSSException e) {
            	e.printStackTrace();
            } 
        
	        GridbusFarmingEngine fe = (GridbusFarmingEngine) session.getAttribute("fe");
	        if (fe != null) {
	        	// move results from brokers temp directory to results directory
	        	File tempDir = new File(fe.getTempDirectory());
	        	File resultsDir = 
	        		new File(getPortletConfig().getPortletContext().getRealPath(
	        				"/files/results")+"/"+fe.getBrokerID());

	        	try {
		        	resultsDir.mkdirs();
	        	
		        	File[] files = tempDir.listFiles();
		        	for (int i = 0; i < files.length; i++) {
		        		if (!files[i].getName().endsWith(".sh") && !files[i].getName().endsWith(".summ")){
			        	    try{
				        		File newFile = new File(resultsDir.getAbsolutePath()+"/"+files[i].getName());
				        		BufferedInputStream bis = new BufferedInputStream(new FileInputStream(files[i]));
				        		BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(newFile));
				        		int temp = 0;
				        		while ((temp=bis.read())!=-1)
				        		    bos.write(temp);
				        		bis.close();bos.close();
				        		//dont move the files anymore. just copy as above
			        	    }catch (Exception e){}
			        		//ps.println("copying "+files[i]+" to "+newFile);

		        		}
		        	}
		        	// display results
	        	} catch (Exception e) {
	        		e.printStackTrace();
	        	}

	        	request.setAttribute("brokerID", fe.getBrokerID());
	    		request.setAttribute("results", resultsDir.list());
	        }
        }

        request.setAttribute("url", response.createActionURL().toString());
        request.setAttribute("userCredName", user);

        getPortletContext().getRequestDispatcher("/jsp/results.jsp").include(request, response);
    }
}