/*
 * Title            :  GetMyProxy.java
 * Package      	:  examples
 * Project        	:  Gridbus Broker
 * Description   	:		
 * Created on   	:  March 31, 2005
 * Author	     	:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright     	:   (c) 2005, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.broker.examples.portlets.gridbusbroker;

import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.globus.myproxy.MyProxy;
import org.gridbus.broker.farming.globus.GridProxyUtil;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;

/**
 * This is a simple portlet for retrieving proxies from MyProxy servers.
 *
 */
public class GetMyProxy extends GenericPortlet {
    
    /**
     * Reads the following form parameters and uses this information to retrieve
     * the users' proxy and then stores it in the session.
     * 
     * username : username under which the user has stored their proxy in the MyProxy server
     * password : password for the proxy in the MyProxy server
     * host : hostname for the MyProxy server at which the user has a proxy stored
     * port : port for the MyProxy server
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public void processAction(ActionRequest request, ActionResponse response)
    	throws PortletException, IOException {

    	// get the session
        PortletSession session = request.getPortletSession(true);

        // get parameters from the form
        String username = request.getParameter("username");
        String password = request.getParameter("password");
        String host = request.getParameter("host");
        String port = request.getParameter("port");

        GSSCredential cred = null;
        MyProxy myproxy = null;
		
        if (request.getParameter("action")!=null && request.getParameter("action").equals("load")){
	        try {
	        	// connect to myproxy server.
	        	myproxy = new MyProxy(host, Integer.parseInt(port));
	        	// get the credential for this user.
	            cred = myproxy.get(username, password, 3600);
	            // store credential in session in application scope so other portlets can use it.
	            session.setAttribute("userCred", cred, PortletSession.APPLICATION_SCOPE);
	        } catch (Exception e) {
	        	e.printStackTrace();
	        }
        }else if (request.getParameter("cmdLocalproxy")!=null && request.getParameter("cmdLocalproxy").equals("generate proxy")){
        	try{
        		GridProxyUtil.setPwd(request.getParameter(""));
        		cred = GridProxyUtil.getProxy();
	            session.setAttribute("userCred", cred, PortletSession.APPLICATION_SCOPE);
        	}catch (Exception e){
        		e.printStackTrace();
        	}
        }
    }
    
    /**
     * Set up any required view parameters.
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public void doView(RenderRequest request, RenderResponse response)
    	throws PortletException, IOException {

    	// just checking to see if the user has loaded their proxy yet.
        PortletSession session = request.getPortletSession(true);
        GSSCredential credential = (GSSCredential) session.getAttribute("userCred");
        String user = "";
        if (credential != null) {
        	try {
        		user = credential.getName().toString();
        	} catch (GSSException e) {
            	e.printStackTrace();
            } 
        }else{
            
        }
        request.setAttribute("userdir", System.getProperty("user.home"));
        request.setAttribute("userCredName", user);
        request.setAttribute("url", response.createActionURL().toString());

        getPortletContext().getRequestDispatcher("/jsp/myproxy.jsp").include(request, response);
    }
}
