package plan2run.ParamObject;


/**
 * Title: GridBus Parameterization Toolkit
 * Description:
 * Copyright: Copyright (c) 2002
 * Company: GRIDS Lab University of Melbourne
 * @author			stevexm@cs.mu.oz.au Steve Melnikoff  Copyright Oct. 2002
 * @version 2.0
 */

import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Iterator;
import java.io.Serializable;
import java.text.NumberFormat;
import java.text.*;
import java.util.*;

public class ParamObject implements Cloneable, Serializable
{
	protected static final String NEWPARAM = "_NEWPARAM_";	// for a user defined parameter
	protected static final String SELECTEDPARAM = "_SELECTEDPARAM_";	// for a user selected parameter
	protected static String objType = SELECTEDPARAM;
	private static final int UNDEFINED = -1;

	public static final String emptyString = "";
	protected  String header = "";
	protected  String footer = "";
	protected static int			headerLength;
	protected static int			footerLength;
	protected static String nameUnmodified = "";
	private static final String planHeader = "parameter";
	private static final String SEMICOLON = ";";
	private static final String planDelimiter = ";";
	private static final String spacer = " ";
	private static final String DBLQUOTE = "\"";
	private static final String bracketHeader = "${";
	private static final String bracketFooter = "}";
	private static final String noBracketHeader = "$";
	private static final String noBracketFooter = "";
	public static final String[] parameterTypes = {"integer","float","text","file","comment"}; // type
	public static final String[] domainTypes = {"single value","range","select anyof","select oneof","random","compute","JITP"}; // type2
	public static final String[] stepTypes = {"step","points"}; // type3
	public static final String DEFAULTSTRING 	= "default";
        public static final String JITPSTRING	        = domainTypes[6];
	public static final String COMPUTESTRING	= domainTypes[5];
	public static final String RANDOMSTRING 	= domainTypes[4];
	public static final String ONEOFSTRING 		= domainTypes[3];
	public static final String ANYOFSTRING 		= domainTypes[2];
	public static final String RANGESTRING 		= domainTypes[1];
	public static final String SINGLESTRING 	= domainTypes[0]; //"single value"
	public static final String INTEGERSTRING 	= parameterTypes[0];
	public static final String FLOATSTRING 		= parameterTypes[1];
	public static final String TEXTSTRING 		= parameterTypes[2];
	public static final String FILESTRING 		= parameterTypes[3];
	public static final String COMMENTSTRING 	= parameterTypes[4];
	public static final String STEPSTRING 		= stepTypes[0];
	public static final String POINTSSTRING = stepTypes[1];
	private static final String EMPTYSTRING = "";
	private static final String HASHSTRING 	= "#";
	private static final String rangeType 	= RANGESTRING;
	private static final String defaultType = SINGLESTRING; //"single value"
	private static final String textType 	= TEXTSTRING;
	private static final String type1default = INTEGERSTRING;
	private static final String type2default = SINGLESTRING; //"single value"
	private static final String type3default = STEPSTRING;
        private static final int    MAX_FRAC = 3;   // max digits for decimal float [ used in makeRunStep() ]

	// Run step strings
	protected static final String VARIABLESTR 	= "variable";
	protected static final String CONSTANTSTR 	= "constant";
	protected static final String INDEXSTR		= "index";
	protected static final String LISTSTR		= "list";

	private Object 		parentFrame;
	private Object 		parentFile;

	protected String	name;
	protected String	parameterName;
	protected String	label;
	protected String	input;
	protected String	type;  // parameter types
	protected String	type2; // domain types
	protected String	type3; // step types
	protected int 		typeIndex;
	protected int 		type2Index;
	protected int 		type3Index;
	protected boolean 	needToBracket;
//	values for type
	protected String	rangeFrom;
	protected String	rangeTo;
	protected String	defaultValue;
	protected String	step;
	protected String	textValue;
	protected String	parameter;
	protected String	planStep;
	protected String	runStep = EMPTYSTRING;
	protected Vector	SelectionValuesVect = new Vector(); // to store values for "select any/oneof-param"
	protected Vector  	DefaultValuesVect = new Vector(); // to store the default values of anyof-param
	protected Vector	allValsVect = null;
	protected Vector	SelectedValues;

	protected boolean 	isComment;
	protected boolean 	assigned = false;
	protected ParamObject undoParam;
	protected ParamObject redoParam;
// constructor
	public ParamObject()
	{
		name = emptyString;
		parameterName = emptyString;
		label = emptyString;
		input = emptyString;
		type = type1default;
		type2 = type2default;
		type3 = type3default;
		rangeFrom = emptyString;
		rangeTo = emptyString;
		defaultValue = emptyString;
		step = emptyString;
		textValue = emptyString;
		parameter = emptyString;
		undoParam = null;
		redoParam = null;
		nameUnmodified = emptyString;
		needToBracket = false;
		planStep = emptyString;
	}

	public	ParamObject(String s)
	{
		if( NEWPARAM.equals(emptyString))	// check if user defined parameter
			this.objType = NEWPARAM;
		name = emptyString;
		parameterName = emptyString;
		label = emptyString;
		input = emptyString;
		type = type1default;
		type2 = type2default;
		type3 = type3default;
		rangeFrom = emptyString;
		rangeTo = emptyString;
		defaultValue = s;
		step = emptyString;
		textValue = emptyString;
		setParameter(emptyString);
		undoParam = null;
		redoParam = null;
		nameUnmodified = s;
		needToBracket = false;
		planStep = emptyString;
	}

	public	ParamObject(String s, boolean needToBracket)
	{
		if( NEWPARAM.equals(emptyString) )	// check if user defined parameter
			this.objType = NEWPARAM;
		name = emptyString;
		parameterName = emptyString;
		label = emptyString;
		input = emptyString;
		type = type1default;
		type2 = type2default;
		type3 = type3default;
		rangeFrom = emptyString;
		rangeTo = emptyString;
		defaultValue = s;
		step = emptyString;
		textValue = emptyString;
		setParameter(emptyString);
		undoParam = null;
		redoParam = null;
		nameUnmodified = s;
		this.needToBracket = needToBracket;
		planStep = emptyString;
	}

	public ParamObject(Object in)
	{
		this.copy(in, this);
	}

	// cloning
	public Object clone()
	{
		try
		{
			return super.clone();
		}
		catch  (CloneNotSupportedException e)
		{
			return null;
		}
	}

	public static void copy(Object in, Object out)
	{
		ParamObject inParam = (ParamObject)in;
		ParamObject outParam = (ParamObject)out;
		if(in != null && out!=null) {
			outParam.name = inParam.name;
			outParam.parameterName = inParam.parameterName;
			outParam.label = inParam.label;
			outParam.input = inParam.input;
			outParam.type = inParam.type;
			outParam.type2 = inParam.type2;
			outParam.type3 = inParam.type3;
			outParam.typeIndex = inParam.typeIndex;
			outParam.type2Index = inParam.type2Index;
			outParam.type3Index = inParam.type3Index;
			if(!outParam.isAssigned())
			{
				outParam.rangeFrom = inParam.rangeFrom;
				outParam.rangeTo = inParam.rangeTo;
			}
			outParam.defaultValue = inParam.defaultValue;
			outParam.step = inParam.step;
			outParam.textValue = inParam.textValue;
			outParam.parameter = inParam.parameter;
			outParam.makePlanStep();
			outParam.undoParam = inParam.undoParam;
			outParam.redoParam = inParam.redoParam;
			outParam.nameUnmodified = inParam.nameUnmodified;
		}
	}

// getters/setters
	public String getName() { return name; }
	public void		setName(String s) { name = s; }
	public String getNameUnmodified() { return nameUnmodified; }

	public String getParameterName() { return parameterName; }
	public void	setParameterName(String s) { parameterName = s; }

	public String getLabel()
	{
		if(label.length() > 0) return label;
		else return parameterName;
	}
	public void		setLabel(String s) { label = removeDblQuote(s); }

	public String getInput() { return input; }
	public void		setInput(String s) { input = s; }

	public String getType() { return type; }
	public void		setType(String s) { type = s; }
	public String getType2() { return type2; }
	public void		setType2(String s) { type2 = s; }
	public String getType3() { return type3; }
	public void		setType3(String s) { type3 = s; }

	public int getTypeIndex() { return typeIndex; }
	public void		setTypeIndex(int i) { typeIndex = i; }
	public int getType2Index() { return type2Index; }
	public void		setDomainIndex(int i) { type2Index = i; }
	public int getType3Index() { return type3Index; }
	public void		setType3Index(int i) { type3Index = i; }

	public String getRangeFrom() { return rangeFrom; }
	public void		setRangeFrom(String s) { rangeFrom = s; }

	public String getRangeTo() { return rangeTo; }
	public void		setRangeTo(String s) { rangeTo = s; }

	public String getDefaultValue() { return defaultValue; }
	public void		setDefaultValue(String s) { defaultValue = s; }

	public String getStep() { return step; }
	public void		setStep(String s) { step = s; }

	public String getTextValue() { return textValue; }
	public void		setTextValue(String s) { textValue = s; }

	public int nameLength() { return name.length() + headerLength + footerLength; }

	public Object getParentFrame() { return parentFrame; }
	public void setParentFrame(Object parent) { parentFrame = parent; }

	public Object getParentFile() { return parentFile; }
	public void setParentFile(Object parent) { parentFile = parent; }

	public boolean isComment() { return isComment; }
	public void setIsComment(boolean f) { isComment = f; }

// handle param file headers & footers
	public 	void	setHeader(String s)
	{
		header = s;
		headerLength = header.length();
 	}
	public 	void	setFooter(String s)
	{
		footer = s;
		footerLength = footer.length();
 	}
// the parameter is equal to the name concat with header and footer
	public void	setNeedToBracket(boolean pb) { needToBracket = pb; }
	public boolean getNeedToBracket() {
		return true;
//		return needToBracket; //always return true
	}

	public void setHeaderFooter() {
		if(getNeedToBracket()) {
			header = bracketHeader;
			footer = bracketFooter;
		} else {
			header = noBracketHeader;
			footer = noBracketFooter;
		}
	}

	public void setParameter() {
		setHeaderFooter();
		parameter = header + parameterName + footer;
	}

	public  String makeParameter(String s) {
		setHeaderFooter();
		return header + s + footer;
	}

	public void setParameter(String s) {
		setHeaderFooter();
		parameter = header + s + footer;
		parameterName = s;
	}

	public String	getParameter() { return parameter; }

        private String removeDblQuote(String s)
        {
          if(s.startsWith(DBLQUOTE) && s.endsWith(DBLQUOTE))
            s = s.substring(1, s.length()-1);
          return s;
        }

// 	plan file processing
	public void	setPlanStep(String s) { planStep = s; }
	public String getPlanStep() { return planStep; }
	public void makePlanStep()
	{
		String planStepSegment;
		isComment = false;
		planStep = planHeader;
		planStep = planStep + spacer + parameterName;
		if(label.length()>0)
			planStep = planStep + spacer + DBLQUOTE + label + DBLQUOTE;
		else
			planStep = planStep + spacer + DBLQUOTE + parameterName + DBLQUOTE;

		planStep = planStep + spacer + type;
		planStepSegment = EMPTYSTRING;

		if(type2.equals(SINGLESTRING) )
		{
			if( type.equals(TEXTSTRING) ) //need "<value>"
				planStepSegment = DEFAULTSTRING + spacer + DBLQUOTE + defaultValue + DBLQUOTE;
			else	// just <value>
				planStepSegment = DEFAULTSTRING + spacer + defaultValue;
		}
		else if( type2.equals(RANGESTRING) )
		{
			planStepSegment = RANGESTRING;
			if(rangeFrom.length()>0)
				planStepSegment = planStepSegment + spacer + "from" + spacer + rangeFrom + spacer + "to" + spacer + rangeTo;
			if(step.length() > 0)
				planStepSegment = planStepSegment + spacer + type3 + spacer + step ;
		} else if( type2.equals(RANDOMSTRING) )
		{
			planStepSegment = RANDOMSTRING;
			if(rangeFrom.length()>0)
				planStepSegment = planStepSegment + spacer + "from" + spacer + rangeFrom + spacer + "to" + spacer + rangeTo;
			if(step.length() > 0)
				planStepSegment = planStepSegment + spacer + type3 + spacer + step ;
		}
		else if( type2.equals(ANYOFSTRING) )
		{
			if( type.equals(TEXTSTRING) )
				planStepSegment = ANYOFSTRING + spacer + tokenize(textValue, false) +
	 					DEFAULTSTRING + spacer + tokenize(defaultValue, true);
	 		else
				planStepSegment = ANYOFSTRING + spacer + tokenize(textValue, false) +
						DEFAULTSTRING + spacer + tokenize(defaultValue, true);
		}
		else if( type2.equals(ONEOFSTRING) )
		{
			DefaultValuesVect.add(defaultValue);
			if( type.equals(TEXTSTRING) )
                          if(defaultValue.startsWith(DBLQUOTE))
                          	planStepSegment = ONEOFSTRING + spacer + tokenize(textValue, false) +
	 					DEFAULTSTRING + spacer + defaultValue;
                          else
				planStepSegment = ONEOFSTRING + spacer + tokenize(textValue, false) +
	 					DEFAULTSTRING + spacer + DBLQUOTE + defaultValue + DBLQUOTE;
	 		else
				planStepSegment = ONEOFSTRING + spacer + tokenize(textValue, false) +
						DEFAULTSTRING + spacer + defaultValue;
		}
		else
		{
			planStepSegment = type2 + spacer + textValue;
			if(defaultValue.length() > 0) planStepSegment = planStepSegment + spacer + DEFAULTSTRING + spacer + defaultValue;
		}


		planStep = planStep + spacer + planStepSegment;
		planStep = planStep + planDelimiter;
		if(type == COMMENTSTRING || textValue.indexOf(HASHSTRING) == 0)
		{
			isComment = true;
			planStep = textValue;
			if(planStep.indexOf(HASHSTRING)==-1) planStep = HASHSTRING + planStep;
		}
	}


//  remove white spaces from the comma separated text
// 	Put the token into vectors to use for building
//	components in ParamPanel
	public String tokenize(String s, boolean isDefaultList)
	{
		int index;
		Vector vect;

		// select the correct Vector to add token values to:
		if(isDefaultList)
		{
			DefaultValuesVect.clear();
			vect = DefaultValuesVect;
		}
		else
		{
			SelectionValuesVect.clear();
			vect = SelectionValuesVect;
		}

		String tkzString = emptyString;
		String nextTkn	= emptyString;
		StringTokenizer tkz = new StringTokenizer(s, ",");
		while(tkz.hasMoreTokens())
		{
			nextTkn = tkz.nextToken();
			for(index=0 ; nextTkn.charAt(index) == ' ' ; index++) // skip all whtspcs after a comma
			System.out.println(nextTkn);
			if(0 != index) // substring takne the number of chars, not 0-(n-1) index
				nextTkn = nextTkn.substring(index);
			//System.out.println(nextTkn);

			vect.add(nextTkn); // to ensure all the selection components in swing take the correct value
			if(type.equals(TEXTSTRING))
                            if(nextTkn.startsWith(DBLQUOTE))
                                tkzString = tkzString + nextTkn + spacer;
                            else
				tkzString = tkzString + DBLQUOTE + nextTkn + DBLQUOTE + spacer;
			else
                            tkzString = tkzString + nextTkn + spacer;

		}
		return tkzString;
	}


// handle undo/redo linked list
	public void setUndoParam(Object paramObj) { undoParam = (ParamObject)paramObj; }
	public Object getUndoParam() { return undoParam; }
	public void setRedoParam(Object paramObj) { redoParam = (ParamObject)paramObj; }
	public Object getRedoParam() { return redoParam; }

// depending on the domain type of the paramObject we return the
// number of values that this parameter object will take-on during run.
	public int getValueCount()
	{
		if(type2.equals(SINGLESTRING))
		{
			if(defaultValue.equals(emptyString)) return UNDEFINED;
				else return 1;
		}
		if(this.type2.equals(RANGESTRING))
		{
			if(null == rangeTypeChecking()) return UNDEFINED;
			if(type3.equals(POINTSSTRING)) return Integer.parseInt(step);
			if(type3.equals(STEPSTRING)) return(calcRangeCount());
		}
		if(type2.equals(ONEOFSTRING))
		{
			if (defaultValue.equals(emptyString)) return UNDEFINED;
			else return 1;
		}
		// ***to be added.
		if(type2.equals(ANYOFSTRING))
		{
			if (defaultValue.equals(emptyString)) return UNDEFINED;
			else return getSelectedValuesCount();
		}
		return UNDEFINED;
	}

	public int getDefaultValuesCount()
	{
		return DefaultValuesVect.size();
	}

	public Vector getDefaultValuesVect()
	{
		return DefaultValuesVect;
	}

	public void setDefaultValuesVect(Vector v)
	{
		DefaultValuesVect = v;
	}

	// get the numeric number of parameter for a range paramObject.
	protected int calcRangeCount()
	{
		int count;
		if ( (Double.parseDouble((rangeTo)) - Double.parseDouble(rangeFrom) ) < 0) return UNDEFINED;
		if ( (Double.parseDouble((rangeTo)) - Double.parseDouble(rangeFrom) ) < (Double.parseDouble(step))) return UNDEFINED;
		count = (int)Math.ceil(((Double.parseDouble((rangeTo)) - Double.parseDouble(rangeFrom))+1)/(Double.parseDouble(step)));
		return count;
	}

	protected Object rangeTypeChecking()
	{
		if(rangeTo.equals(emptyString)) return null;
		if(rangeFrom.equals(emptyString)) return null;
		if(step.equals(emptyString)) return null;
		if(type.equals(INTEGERSTRING))
		{
			try
			{
				Integer.parseInt(rangeTo);
				Integer.parseInt(rangeFrom);
				Integer.parseInt(step);
			}
			catch (java.lang.NumberFormatException e)
			{
				return null;
			}
		}
		if(type.equals(FLOATSTRING))
		{
			try
			{
				Double.parseDouble(rangeTo);
				Double.parseDouble(rangeFrom);
				Double.parseDouble(step);
			}
			catch (java.lang.NumberFormatException e)
			{
				return null;
			}
		}
		return new Object();

	}

	// used to obtain all the range values in a vector
	public Vector getRangeValues()
	{
		if(null == rangeTypeChecking()) return null;

		int count = calcRangeCount();
		if(UNDEFINED == count) return null;

		double dVal; // Floats
		int iVal;	// Integers
		Vector rangeVals = new Vector();

		if(type2.equals(RANGESTRING))
		{	//step
			if(type.equals(FLOATSTRING))
			{
				dVal = Double.parseDouble(rangeFrom);
                                NumberFormat dblFormatter;
                                dblFormatter = NumberFormat.getNumberInstance();
                                dblFormatter.setMaximumFractionDigits(MAX_FRAC);
				for (int i=0 ; i<count ; i++)
				{
                                        Double d = new Double(dVal);
                                        rangeVals.add(dblFormatter.format(d));
                                        dVal += Double.parseDouble(step);
				}
			}
			else if(type.equals(INTEGERSTRING))
			{
				iVal = Integer.parseInt(rangeFrom);
				for (int i=0 ; i<count ; i++)
				{
					rangeVals.add(Integer.toString(iVal));
					iVal += Integer.parseInt(step);
				}
			}
		}
		else if(type2.equals(POINTSSTRING))
		{	// points
			System.out.println("-***-points-***-");
		}

		return rangeVals;
	}

	// to see if all requied fields are defined.
	public boolean isDefined()
	{
		if (this.getValueCount() == UNDEFINED)
			return false;
		else
			return true;
	}

	public Vector getSelectValuesVect()
	{
		return SelectionValuesVect;
	}

	public void makeRunStep(int listIndex)
	{
		runStep = VARIABLESTR + spacer + getParameterName() + spacer; //variable <name>

		if(type2.equals(SINGLESTRING))
		{
			runStep += CONSTANTSTR + spacer + DBLQUOTE + getDefaultValue() + DBLQUOTE + SEMICOLON;
		}	//variable <name> constant "<value>";

		else if(type2.equals(RANGESTRING))
		{	//variable <name> index <n> list <list>
			runStep += INDEXSTR + spacer + listIndex + spacer +
				LISTSTR + spacer + getListString() + SEMICOLON;
		}

		else if(type2.equals(ANYOFSTRING))
		{
			runStep += INDEXSTR + spacer + listIndex + spacer +
				LISTSTR + spacer + getListString() + SEMICOLON;
		}

		else if(type2.equals(ONEOFSTRING))
		{
			runStep += CONSTANTSTR + spacer + getListString() + SEMICOLON;
		}

		else if(type2.equals(COMPUTESTRING))
		{

		}

		else if(type2.equals(RANDOMSTRING))
		{

		}

	}

	public String getRunStep() { return runStep; }

	// creates a string of values as they
	// will appear in the runfile.
	protected String getListString()
	{
		Vector valVect = null;
		String list = EMPTYSTRING;

		if ( type2.equals(RANGESTRING))
		{
			valVect = getRangeValues();
		}
		else if ( type2.equals(ANYOFSTRING) || type2.equals(ONEOFSTRING))
		{
			valVect = getSelectedValuesVect();
		}

		if(null != valVect)
		{
			for(int i=0 ; i<valVect.size() ; i++)
			{
                            if( ((String)valVect.elementAt(i)).startsWith(DBLQUOTE) )
                              list += valVect.elementAt(i) + spacer;
                            else
                              list += DBLQUOTE + valVect.elementAt(i) + DBLQUOTE + spacer;
                        }
			return list;
		}
		return null;
	}

	public boolean isAssigned() { return assigned; }

	public void setIsAssigned(boolean bool) { assigned = bool; }

	// all Selectable values are in places by taking a UNION of
	// SelectionValuesVect AND DefaultValuesVect and REMOVING
	// the INTERSECTION of SelectionValuesVect AND DefaultValuesVect
	public void setAllSelectValuesVect()
	{
		allValsVect = new Vector(SelectionValuesVect);
		Iterator i = DefaultValuesVect.iterator();
		Object val;
		while(i.hasNext())
		{
			val = i.next();
			if(!allValsVect.contains(val))
				allValsVect.add(val);
		}
	}

	// all Selectable values are in here
	public void setAllSelectValuesVect(Vector v)
	{
		allValsVect = v;
	}

	// the values selected at the run file generation are set using this method
	public void setSelectedValues(Vector v)
	{
		SelectedValues = v;
	}

	// get all the values selected at run file generation step
	public Vector getSelectedValuesVect()
	{
		return SelectedValues;
	}

	public int getSelectedValuesCount()
	{
		if(null == SelectedValues) return getDefaultValuesCount();
		return SelectedValues.size();
	}

	// get all the selectable values;
	public Vector getAllSelectValuesVect()
	{
		if (null == allValsVect) setAllSelectValuesVect();
		return allValsVect;
	}


}
